
 #include <math.h>
/*********************************************************
  Funciones para hallar la distancia entre dos puntos

  Parametros:
                punto A
		punto B
  Retorno:
                valor de la distancia
 ********************************************************/

float m_distance3D(float *v1, float *v2)
 {
     float v[3];

     v[0] = v1[0]-v2[0];
     v[1] = v1[1]-v2[1];
     v[2] = v1[2]-v2[2];
     return((float)sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]));
 }

float m_distance2D(float x1, float y1, float x2, float y2)
 {
     float v[2];

     v[0] = x1-x2;
     v[1] = y1-y2;
     return((float)sqrt(v[0]*v[0]+v[1]*v[1]));
 }

/*********************************************************
  Funcion para convertir de coordenadas 3D a polares

  Parametros:
                punto en 3D
		centro de coordenadas relativo
		referencia a coordenada radial
		referencia a angulo x
		referencia a angulo y
  Retorno:
                ninguno
 ********************************************************/

void m_3D_to_polar(float *p, float *c,
  float *ref_R, float *ref_X, float *ref_Y)
{
   float r;

   r = m_distance3D(c, p);
   *ref_R = r;
   *ref_X = (float)(atan((p[2]-c[2])/(p[0]-c[0])));
   *ref_Y = (float)(asin(m_distance2D(p[0],p[2],c[0],c[2])/r));
}

/*********************************************************
  Funcion para convertir de coordenadas polares a 3D

  Parametros:
                coordenada radial
		angulo x
		angulo y
		centro de coordenadas relativo
		punto en 3D (resultado)
  Retorno:
                ninguno
 ********************************************************/

void m_polar_to_3D(float r, float x, float y, float *c, float *p)
{
   float sex, cox, sey, coy;

   sex = (float)sin(x);
   cox = (float)cos(x);
   sey = (float)sin(y);
   coy = (float)cos(y);

   p[0] = c[0]+r*cox*sey;
   p[1] = c[1]+r*coy;
   p[2] = c[2]+r*sex*sey;
}

/*********************************************************
  Funcion para calcular modulo de vectores 3D

  Parametros:
                vector 3D
  Retorno:
                modulo
 ********************************************************/

float m_module3D(float *v)
{
   return((float)sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]));
}

/*********************************************************
  Funcion para normalizar vectores 3D

  Parametros:
                vector a normalizar
  Retorno:
                ninguno
 ********************************************************/

void m_normalize3D(float *v)
{
   float m;

   m = 1/m_module3D(v);
   v[0] *= m;
   v[1] *= m;
   v[2] *= m;
}

/*********************************************************
  Funcion para normalizar vectores 2D

  Parametros:
                vector a normalizar
  Retorno:
                ninguno
 ********************************************************/
 void m_normalize2D(float *v)
{
   double m;

   m = 1/sqrt(v[0]*v[0]+v[1]*v[1]);
   v[0]*=m;
   v[1]*=m;
}

/*********************************************************
  Funcion para hallar vector diferencia (v2-v1) normalizado

  Parametros:
                vector1
		vector2
		vector diferencia normalizado (resultado)
  Retorno:
                ninguno
 ********************************************************/

void m_norm_diference3D(float *v1, float *v2, float *r)
{
   r[0] = v2[0]-v1[0];
   r[1] = v2[1]-v1[1];
   r[2] = v2[2]-v1[2];
   m_normalize3D(r);
}

/*********************************************************
  Funcion para hallar vector diferencia (v2-v1)

  Parametros:
                vector1
		vector2
		vector diferencia (resultado)
  Retorno:
                ninguno
 ********************************************************/

void m_diference3D(float *v1, float *v2, float *r)
{
   r[0] = v2[0]-v1[0];
   r[1] = v2[1]-v1[1];
   r[2] = v2[2]-v1[2];
}
              /*********************************************************
  Funcion para hallar producto vectorial de dos vectores 3D

  Parametros:
                vector1
		vector2
		vector resultado
  Retorno:
                ninguno
 ********************************************************/


void m_vector_prod_3D(float *v1, float *v2, float *r)
{
   r[0] = v1[1]*v2[2]-v1[2]*v2[1];
   r[1] = v1[2]*v2[0]-v1[0]*v2[2];
   r[2] = v1[0]*v2[1]-v1[1]*v2[0];
}
   /*********************************************************
  Funcion para hallar producto escalar de dos vectores 3D

  Parametros:
                vector1
		vector2
  Retorno:
                producto escalar
 ********************************************************/

float m_dot_prod_3D(float *v1, float *v2)
{
   return(v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2]);
}



